"""Defines how connections with databases via peewee will be handled"""
import hug
from peewee import MySQLDatabase, PostgresqlDatabase, SqliteDatabase, Model
from playhouse.berkeleydb import BerkeleyDatabase
from playhouse.shortcuts import model_to_dict

ENGINES = {'sqlite': SqliteDatabase, 'psql': PostgresqlDatabase, 'mysql': MySQLDatabase, 'berkeley': BerkeleyDatabase}


def manage(api, engine='sqlite', location=':memory:', **settings):
    '''Creates a new database object for handling connections based on the specified parameters'''
    if not engine in ENGINES:
        raise ValueError(('{0} is not one of the registered database engines: {1}.'
                         "To add an engine: hug_peewee.ENGINES['{0}'] = Class").format(engine,
                                                                                        ', '.join(ENGINES.keys())))
    engine_instance = ENGINES[engine](location, **settings)
    if location != ':memory:':
        hug_api = hug.API(api)
        @hug.request_middleware(api=hug_api)
        def process_data(request, response):
            engine_instance.connect()

        @hug.response_middleware(api=hug_api)
        def process_data(request, response, resource):
            engine_instance.close()

    class DatabaseModel(Model):
        class Meta:
            database = engine_instance

        def __native_types__(self):
            return model_to_dict(self)

        @classmethod
        def row(cls, primary_key='id'):
            def get_item(id):
                if isinstance(id, cls):
                    return id

                return cls.get(getattr(cls, primary_key)==id)
            get_item.__doc__ = 'A {} retrieved based on unique {}'.format(cls.__name__, primary_key)
            return get_item

    engine_instance.Model = DatabaseModel
    return engine_instance



#!/usr/bin/env python
"""Defines the setup instructions for hug_peewee"""
import glob
import os
import subprocess
import sys
from os import path

from setuptools import Extension, find_packages, setup
from setuptools.command.test import test as TestCommand

MYDIR = path.abspath(os.path.dirname(__file__))
JYTHON = 'java' in sys.platform
PYPY = bool(getattr(sys, 'pypy_version_info', False))
CYTHON = False
if not PYPY and not JYTHON:
    try:
        from Cython.Distutils import build_ext
        CYTHON = True
    except ImportError:
        pass

cmdclass = {}
ext_modules = []
if CYTHON:
    def list_modules(dirname):
        filenames = glob.glob(path.join(dirname, '*.py'))

        module_names = []
        for name in filenames:
            module, ext = path.splitext(path.basename(name))
            if module != '__init__':
                module_names.append(module)

        return module_names

    ext_modules = [
        Extension('hug_peewee.' + ext, [path.join('hug_peewee', ext + '.py')])
        for ext in list_modules(path.join(MYDIR, 'hug_peewee'))]
    cmdclass['build_ext'] = build_ext


class PyTest(TestCommand):
    extra_kwargs = {'tests_require': ['pytest', 'mock']}

    def finalize_options(self):
        TestCommand.finalize_options(self)
        self.test_args = []
        self.test_suite = True

    def run_tests(self):
        import pytest
        sys.exit(pytest.main(self.test_args))


cmdclass['test'] = PyTest

try:
   import pypandoc
   readme = pypandoc.convert('README.md', 'rst')
except (IOError, ImportError, OSError, RuntimeError):
   readme = ''

setup(name='hug_peewee',
      version='0.0.2',
      description='An extension to hug that adds clean integration with the peewee ORM',
      long_description=readme,
      author='Timothy Crosley',
      author_email='timothy.crosley@gmail.com',
      url='https://github.com/timothycrosley/hug_peewee',
      license="MIT",
      packages=['hug_peewee'],
      requires=['hug', 'peewee'],
      install_requires=['hug', 'peewee'],
      cmdclass=cmdclass,
      ext_modules=ext_modules,
      keywords='Python, Python3',
      classifiers=['Development Status :: 6 - Mature',
                   'Intended Audience :: Developers',
                   'Natural Language :: English',
                   'Environment :: Console',
                   'License :: OSI Approved :: MIT License',
                   'Programming Language :: Python',
                   'Programming Language :: Python :: 3',
                   'Programming Language :: Python :: 3.2',
                   'Programming Language :: Python :: 3.3',
                   'Programming Language :: Python :: 3.4',
                   'Programming Language :: Python :: 3.5',
                   'Topic :: Software Development :: Libraries',
                   'Topic :: Utilities'],
      **PyTest.extra_kwargs)



"""Defines how the hug_peewee package exposes modules as well as exposes it's version number"""
from hug_peewee._version import current
from hug_peewee.connection import ENGINES
from hug_peewee import connection

__version__ = current
